PROGRAM deconvolve_mask
  IMPLICIT none
!
! This program uses the so-called MASTER algorithm [Hivon et al.,
! ApJ, 567, 2 (2002)] to deconvolve the power spectrum for the 
! effect of a mask applied to a map. It returns BINNED power spectrum.
!
! The binning is done on cl, which is suitable for the analysis of 
! Poisson-dominated sources such as Gamma-ray data. The output will be
! the binned cl, without the "l(l+1)/twopi" factor.
!
! The input power spectrum file should contain the pseudo power spectrum
! data WITHOUT l(l+1)/twopi factor from l=0 in the following format:
!
! cl at l=0
! cl at l=1
! cl at l=2
! .
! .
!
! The input mask power spectrum and beam transfer function files should 
! also be in the same format. The output file, "multipole_deconvolved_cl.txt", 
! will be in the format:
!
! multipole   deconvolved cl at l=2+(b-1/2)*binsize
! .
! .
! where "b" is the bin number (b=1,2,...) and "binsize" is the size of a bin.
!
! Based on MASTER code from June 27, 2010: E.Komatsu
! April 27, 2013 : R. Flauger
! Calculation of the coupling matrix is modified to use symmetry of the square of Wigner 3j
! July 26, 2018: R. Makiya
! fixed the bug in treatment of b_l (since ver. July 26) February 25, 2019: R. Makiya

  integer :: nlmax,l,binsize,nbmax,b
  double precision, dimension(:), allocatable :: cl,ml,cb,bl
  double precision, dimension(:,:), allocatable :: K
  integer, dimension(:), allocatable :: ipiv
  character(len=256) :: filename1,filename2,filename3
  print*,'NOTE: the input power spectra should not contain l(l+1)/twopi factors!'
  print*,'enter the maximum multipole (eg: 1024):'
  read*,nlmax
  print*,'enter the binsize (eg: 10):'
  read*,binsize
  nbmax=(nlmax-1)/binsize
  allocate(cl(0:nlmax),ml(0:nlmax),bl(0:nlmax),ipiv(1:nbmax),cb(1:nbmax))
  allocate(K(1:nbmax,1:nbmax))
  print*,'enter filename of the power spectrum to be deconvolved (eg: cl_cutsky.txt):'
  read*,filename1
  print*,'enter filename of the power spectrum of the mask (eg: cl_of_mask.txt):'
  read*,filename2
  print*,'enter filename of the beam transfer function (eg: bl.txt):'
  print*,'NOTE: The beam transfer function, b_l, is defined such that the beam-smeared' 
  print*,'      power spectrum is given by C_l[smeared]=C_l[true]*b_l^2. This should'
  print*,'      also include the appropriate pixel-window function.'
  read*,filename3
  open(1,file=filename1,status='old')
  open(2,file=filename2,status='old')
  open(3,file=filename3,status='old')
  do l=0,nlmax
     read(1,*)cl(l)
     read(2,*)ml(l) 
     read(3,*)bl(l) 
  enddo
  close(1)
  close(2)
  close(3)
  print*,'Starting...'
  call compute_coupling_matrix(nlmax,nbmax,binsize,ml,bl,K,ipiv)
  call compute_cl(nlmax,nbmax,binsize,cl,K,ipiv,cb)
  open(4,file='multipole_deconvolved_cl.txt',status='unknown')
  do b=1,nbmax
     write(4,*) 2+binsize*(b-0.5d0)-0.5d0,cb(b)
  enddo
  print*,'Finished. "multipole_deconvolved_cl.txt" contains:'
  print*,'multipole,  binned cl'
  close(4)
END PROGRAM deconvolve_mask
